"""Pool of dummy NTA workers
"""
import zmq
import random
from time import sleep
from threading import Thread
from proto_proto_pscr.cb_psc_nta_pb2 import (
    NtaRequest,
    Flush,
    Configure,
    NtaNetEvent,
    NtaNetEvents,
    LicenseInfo,
    NtaResponse,
)

context = zmq.Context()
REQUEST_TIMEOUT = 3000


def create_socket(nta_id):
    socket = context.socket(zmq.REQ)
    socket.setsockopt_string(zmq.IDENTITY, nta_id)
    socket.connect("tcp://localhost:5555")

    return socket


def close_socket(socket):
    socket.setsockopt(zmq.LINGER, 0)
    socket.close()


def wrap_message(request):
    return NtaRequest(**request)


def generate_random_event_batch(nta_id):

    events = [NtaNetEvent() for _ in range(random.randint(1, 10))]
    subnet_id_hash = abs(hash(nta_id)) % 1000
    subnet_id = LicenseInfo(
        **{"user_id": subnet_id_hash, "access_key_id": subnet_id_hash, "subkey_id": 0}
    )
    events_proto = NtaNetEvents(**{"events": events, "subnet_id": subnet_id})

    return {"nta_net_events": events_proto}


def generate_flush_message(nta_id, terminate=False):
    flush_proto = Flush(**{"terminate": terminate})
    return {"flush": flush_proto}


def generate_configure_message(nta_id):
    configure_proto = Configure()
    return {"configure": configure_proto}


def generate_random_message(nta_id):
    message_builder = random.choice(
        [
            generate_random_event_batch,
            generate_flush_message,
            generate_configure_message,
        ]
    )
    return message_builder(nta_id)


def decode_reply(message):
    reply = NtaResponse()
    reply.ParseFromString(message)
    return reply


def mock_nta_spammer(nta_id):
    # Socket to talk to server
    print(f"Creating mock nta: {nta_id}")
    socket = create_socket(nta_id)

    #  Do 10 requests, waiting each time for a response
    n_requests = 10
    for request in range(n_requests):
        if request == n_requests - 1:
            message = wrap_message(generate_flush_message(nta_id, terminate=True))
        else:
            message = wrap_message(generate_random_message(nta_id))

        message = message.SerializeToString()
        socket.send(message)
        while socket.poll(REQUEST_TIMEOUT) != zmq.POLLIN:
            print("No response, resending")
            close_socket(socket)
            socket = create_socket(nta_id)
            socket.send(message)

        sleep(0.2)
        #  Get the reply.
        reply = socket.recv()
        print(f"NTA {nta_id} received reply: {decode_reply(reply)}")


def main():
    nta_threads = []

    for i in range(10):
        t = Thread(target=mock_nta_spammer, args=(f"nta_{i}",))
        t.start()
        nta_threads.append(t)

    map(lambda x: x.join(), nta_threads)


if __name__ == "__main__":
    main()
